@@ -1,3 +1,4 @@
+
 ###{{{ distribution
 
 ##' @export
@@ -126,7 +127,7 @@
 ##' @export
 normal.lvm <- function(link="identity",mean,sd,log=FALSE,...) {
   rnormal <- if(log) rlnorm else rnorm
-  fam <- gaussian(link); fam$link <- link
+  fam <- stats::gaussian(link); fam$link <- link
   f <- function(n,mu,var,...) rnormal(n,fam$linkinv(mu),sqrt(var))
   if (!missing(mean)) attr(f,"mean") <- mean
   if (!missing(sd)) attr(f,"variance") <- sd^2
@@ -147,7 +148,7 @@ lognormal.lvm <- function(...) normal.lvm(...,log=TRUE)
 
 ##' @export
 poisson.lvm <- function(link="log",lambda,...) {
-    fam <- poisson(link); fam$link <- link
+    fam <- stats::poisson(link); fam$link <- link
     f <- function(n,mu,...) {
         if (missing(n)) {
             return(fam)
@@ -199,7 +200,7 @@ pareto.lvm <- function(lambda=1,...) {   ## shape: lambda, scale: mu
 ##' @export
 threshold.lvm <- function(p,labels=NULL,...) {
     if (sum(p)>1 || any(p<0 | p>1)) stop("wrong probability vector") ;
-    if (!missing(labels))
+    if (!is.null(labels))
     return(function(n,...) {
         return(cut(rnorm(n),breaks=c(-Inf,qnorm(cumsum(p)),Inf),labels=labels))
     })
@@ -213,13 +214,13 @@ threshold.lvm <- function(p,labels=NULL,...) {
 
 ##' @export
 binomial.lvm <- function(link="logit",p,size=1) {
-    fam <- binomial(link); fam$link <- link
+    fam <- stats::binomial(link); fam$link <- link
     f <- function(n,mu,var,...) {
-      if (missing(n)) {
-        return(fam)
-      }
-      rbinom(n,size,fam$linkinv(mu))
-  }
+        if (missing(n)) {
+            return(fam)
+        }
+        rbinom(n,size,fam$linkinv(mu))
+    }
     attr(f,"family") <- fam
     attr(f,"var") <- FALSE
     if (!missing(p)) attr(f,"mean") <- fam$linkfun(p)
@@ -232,7 +233,7 @@ binomial.lvm <- function(link="logit",p,size=1) {
     ##             ### function(n,mu=0,var=1,...) (rnorm(n,mu,sqrt(var))>0)*1
     ##             )
     ##}
-  return(f)
+    return(f)
 }
 
 ##' @export
@@ -248,7 +249,7 @@ probit.lvm <- binomial.lvm("probit")
 
 ##' @export
 Gamma.lvm <- function(link="inverse",shape,rate,unit=FALSE,var=FALSE,log=FALSE,...) {
-  fam <- Gamma(link); fam$link <- link
+  fam <- stats::Gamma(link); fam$link <- link
   rgam <- if (!log) rgamma else function(...) log(rgamma(...))
   if (!missing(shape) & !missing(rate))
     f <- function(n,mu,var,...) rgam(n,shape=shape,rate=rate)
@@ -356,30 +357,42 @@ weibull.lvm <- function(scale=100,shape=2) {
 
 ###{{{ sequence
 ##' @export
-sequence.lvm <- function(a=0,b=1) {
-    f <- function(n,...)
-      seq(a,b,length.out=n)
-  return(f)
+sequence.lvm <- function(a=0,b=1,integer=FALSE) {
+    if (integer) {
+        f <- function(n,...) seq(n)
+        return(f)
+    }
+    if (is.null(a) || is.null(b)) {
+        if (!is.null(a)) {
+            f <- function(n,...) seq(a,length.out=n)
+        } else {
+            f <- function(n,...) seq(n)-(n-b)
+        }
+    } else {
+        f <- function(n,...) seq(a,b,length.out=n)
+    }
+    return(f)
 }
 ###}}} sequence
 
 ###{{{ ones
 ##' @export
-ones.lvm <- function(p=0,interval=NULL) {
+ones.lvm <- function(p=1,interval=NULL) {
     f <- function(n,...) {
         if (!is.null(interval)) {
-            val <- rep(0,n)
+            val <- rep(0L,n)
             if (!is.list(interval)) interval <- list(interval)
             for (i in seq_along(interval)) {
                 ii <- interval[[i]]
                 lo <- round(ii[1]*n)
                 hi <- round(ii[2]*n)
-                val[seq(lo,hi)] <- 1
+                val[seq(lo,hi)] <- 1L
             }
             return(val)
         }
-        val <- rep(1,n)
-        if (p>0 && p<1) val[seq(n*(1-p))] <- 0
+        if (p==0) return(rep(0L,n))
+        val <- rep(1L,n)
+        if (p>0 && p<1) val[seq(n*(1-p))] <- 0L
         val
         }
   return(f)
