@@ -16,7 +16,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     xfix <- c(xfix, list(xfix0))
   }
   if (missing(fix)) {
-    fix <- !any(unlist(lapply(xfix, function(x) length(x)>0)))  
+    fix <- !any(unlist(lapply(xfix, function(x) length(x)>0)))
   }
 
   for (i in seq_len(nm)) {
@@ -28,7 +28,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     }
     if (!lava.options()$exogenous) exogenous(models[[i]]) <- NULL
   }
-  
+
   models.orig <- NULL
 ######################
 ### MLE with MAR mechanism
@@ -40,7 +40,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     mynpar <- c()
     for (i in seq_len(nm)) {
       ## Fix some parameters (predictors,latent variables,...)
-        
+
       d0 <- datasets[[i]][1,,drop=FALSE]; d0[,] <- 1
       if (fix)
         models[[i]] <- fixsome(models[[i]], exo.fix=exo.fix, measurement.fix=fix, data=d0)
@@ -56,12 +56,12 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     pos <- 1
     while(pos<=nonamepar) {
       i <- i+1
-      newname <- paste("par",i,sep="")
+      newname <- paste0("par",i)
       if (!(newname%in%reservedpars)) {
         newpars <- c(newpars,newname)
         pos <- pos+1
       }
-    } 
+    }
 
     pos <- 0
     models0 <- list()
@@ -75,7 +75,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
       mydata <- datasets[[i]][,myvars]
       if (any(is.na(mydata))) {
         if (i>1) pos <- pos+mynpar[i-1]
-        models[[i]] <- baptize(models[[i]],newpars[pos+1:mynpar[i]] ,overwrite=FALSE)        
+        models[[i]] <- baptize(models[[i]],newpars[pos+seq_len(mynpar[i])] ,overwrite=FALSE)
         val <- missingModel(models[[i]],mydata,fix=FALSE,keep=keep,...)
         nmodels <- c(nmodels,length(val$models))
         complidx <- c(complidx,val$pattern.allcomp+nmodels[i]+1)
@@ -95,7 +95,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     }
 
     models.orig <- models
-    
+
     suppressWarnings(val <- multigroup(models0,datasets0,fix=FALSE,missing=FALSE,exo.fix=TRUE,...))
     val$models.orig <- models.orig; val$missing <- TRUE
     val$complete <- complidx-1
@@ -105,7 +105,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     return(val)
   }
 
-  
+
 ######################
 ### Usual analysis:
 ######################
@@ -114,13 +114,13 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     if (inherits(datasets[[i]],c("data.frame","matrix"))) {
       myvars <- intersect(colnames(datasets[[i]]),c(vars(models[[i]]),xfix[[i]],keep))
       if (any(is.na(datasets[[i]][,myvars]))) {
-        if (!warned) warning(paste("Missing data encountered. Going for complete-case analysis", sep=""))
+        if (!warned) warning(paste0("Missing data encountered. Going for complete-case analysis"))
         warned  <- TRUE
         datasets[[i]] <- na.omit(datasets[[i]][,myvars,drop=FALSE])
       }
     }
   }
-    
+
   exo <- exogenous(models)
   means <- lvms <- As <- Ps <- ps <- exs <- datas <- samplestat <- list()
   for (i in seq_len(nm)) {
@@ -134,19 +134,19 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     mydata <- datasets[[i]]
     mymodel <- fixsome(models[[i]], data=mydata, measurement.fix=fix, exo.fix=exo.fix)
     mymodel <- updatelvm(mymodel,zeroones=TRUE,deriv=TRUE)
-    
+
     P <- index(mymodel)$P1; P[P==0] <- NA
     P[!is.na(P) & !is.na(mymodel$covpar)] <- mymodel$covpar[!is.na(P) & !is.na(mymodel$covpar)]
 
     A <- index(mymodel)$M1; A[A==0] <- NA
     A[!is.na(A) & !is.na(mymodel$par)] <- mymodel$par[!is.na(A) & !is.na(mymodel$par)]
-    
+
     mu <- unlist(mymodel$mean)[which(index(mymodel)$v1==1)]
     ex <- names(mymodel$expar)[which(index(mymodel)$e1==1)]
 
     p <- pars(mymodel, A, P, e=ex)
     p[p=="1"] <- NA
-    
+
     means <- c(means, list(mu))
     lvms <- c(lvms, list(mymodel))
     datas <- c(datas, list(mydata))
@@ -163,7 +163,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
   opt <- options(warn=-1); pidx <- is.na(as.numeric(parname)); options(opt)
   parname <- unique(unlist(pp[!is.na(pp)]));
   nfree <- sum(is.na(pp)) + length(parname)
-  
+
   if (nfree>0) {
     pp0 <- lapply(ps, is.na)
     usedname <- cbind(parname, rep(NA,length(parname)))
@@ -174,15 +174,15 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
       for (j in seq_len(length(pp0[[i]]))) {
         pidx <- match(ps[[i]][j],parname)
         if (pp0[[i]][j]) {
-          pres[[i]][j] <- paste("p",counter,sep="")
+          pres[[i]][j] <- paste0("p",counter)
           pres0[[i]][j] <- counter
-          counter <- counter+1        
+          counter <- counter+1
         } else if (!is.na(pidx)) {
           if (!is.na(usedname[pidx,2])) {
             pres[[i]][j] <- usedname[pidx,2]
             pres0[[i]][j] <- as.numeric(substr(pres[[i]][j],2,nchar(pres[[i]][j])))
           } else {
-            val <- paste("p",counter,sep="")
+            val <- paste0("p",counter)
             pres[[i]][j] <- val
             pres0[[i]][j] <- counter
             usedname[pidx,2] <- val
@@ -193,7 +193,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
         }
       }
     }
-    mypar <- paste("p",seq_len(nfree),sep="")
+    mypar <- paste0("p",seq_len(nfree))
     myparPos <- pres0
     myparpos <- pres
     myparlist <- lapply(pres, function(x) x[!is.na(x)])
@@ -205,7 +205,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
   }
 
   ### Mean parameter
-  
+
   mm <- unlist(means)
   meanparname <- unique(mm[!is.na(mm)])
   opt <- options(warn=-1); midx <- is.na(as.numeric(meanparname)); options(opt)
@@ -215,7 +215,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
   ## mean.fixed <- na.omit(match(parname,mm))
   mean.omit <- lapply(means,function(x) na.omit(match(parname,x)))
   nmean <- lapply(means,length)
-  
+
   if (any.mean>0) {
     mm0 <- lapply(means, is.na)
     usedname <- cbind(meanparname, rep(NA,length(meanparname)))
@@ -226,22 +226,22 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
       for (j in seq_len(length(mm0[[i]]))) {
         midx <- match(means[[i]][j],meanparname)
         if (mm0[[i]][j]) {
-          res[[i]][j] <- paste("m",counter,sep="")
+          res[[i]][j] <- paste0("m",counter)
           res0[[i]][j] <- counter
-          counter <- counter+1        
+          counter <- counter+1
         } else if (!is.na(midx)) {
           pidx <- match(meanparname[midx],pp)
           if (!is.na(pidx)) {
             res[[i]][j] <- unlist(myparlist)[pidx]
             res0[[i]][j] <- as.numeric(substr(res[[i]][j],2,nchar(res[[i]][j]))) +
               nfree.mean
-              ##nmean[[i]]            
+              ##nmean[[i]]
           } else {
             if (!is.na(usedname[midx,2])) {
               res[[i]][j] <- usedname[midx,2]
               res0[[i]][j] <- as.numeric(substr(res[[i]][j],2,nchar(res[[i]][j])))
             } else {
-              val <- paste("m",counter,sep="")
+              val <- paste0("m",counter)
               res[[i]][j] <- val
               res0[[i]][j] <- counter
               usedname[midx,2] <- val
@@ -256,18 +256,18 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     mymeanPos <- res0
     mymeanpos <- res
     mymeanlist <- lapply(res, function(x) x[!is.na(x)])
-    mymean <- unique(unlist(mymeanlist))  
+    mymean <- unique(unlist(mymeanlist))
   } else {
     mymeanPos <- NULL
     mymean <- NULL
     mymeanpos <- NULL
     mymeanlist <- NULL
-  }  
+  }
+
+### Extra parameters
 
-### Extra parameters 
-  
   N <- nfree+nfree.mean
-  m0 <- p0 <- c() 
+  m0 <- p0 <- c()
   coefs <- coefsm <- mm0 <- mm <- pp0 <- pp <- c()
   for (i in seq_len(length(myparPos))) {
     mi <- mymeanPos[[i]]
@@ -279,7 +279,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     pp0 <- c(pp0,list(match(p1,pi)))
     if (length(mean.omit[[i]])>0) mi <- mi[-mean.omit[[i]]]
     m1 <- setdiff(mi,m0)
-    m0 <- c(m0,m1)    
+    m0 <- c(m0,m1)
     mm0 <- c(mm0,list(match(m1,mi)))
     pp <- c(pp,list(c(m1,p1+nfree.mean)))
     if (length(p1)>0)
@@ -290,7 +290,7 @@ multigroup <- function(models, datasets, fix, exo.fix=TRUE, keep=NULL, missing=F
     }
   }
   coefs <- c(coefsm,coefs)
-  
+
   res <- list(npar=nfree, npar.mean=nfree.mean,
               ngroup=length(lvms), names=mynames,
               lvm=lvms, data=datas, samplestat=samplestat,
