@@ -1,9 +1,176 @@
+procformula <- function(object=NULL,value,exo=lava.options()$exogenous,...) {
+
+    ## Split into reponse and covariates by ~ disregarding expressions in parantheses
+    ## '(?!...)' Negative lookahead assertion
+    regex <- "~(?![^\\(].*\\))"
+    yx <- lapply(strsplit(as.character(value),regex,perl=TRUE),function(x) gsub(" ","",x))[-1]
+    iscovar <- FALSE
+    if (length(yx)==1) {
+        lhs <- NULL; xidx <- 1
+    } else {
+            lhs <- yx[1]; xidx <- 2
+            if (yx[[xidx]][1]=="") {
+                yx[[xidx]] <- yx[[xidx]][-1]
+                iscovar <- TRUE
+            }
+    }
+    ##Check for link function
+    invlink <- NULL
+    if (xidx==2) {
+        if (length(grep("[a-zA-Z0-9_]*\\(.*\\)$",yx[[xidx]]))>0) { ## rhs of the form F(x+y)
+            invlink <- strsplit(yx[[xidx]],"\\(.*\\)")[[1]][1]
+                if (invlink%in%c("f","v","I","") ||
+                    grepl("+",invlink))
+                { ## Reserved for setting linear constraints
+                    invlink <- NULL
+                } else {
+                    yx[[xidx]] <- gsub(paste0(invlink,"\\(|\\)$"),"",yx[[xidx]])
+                }
+            }
+    }
+
+    ## Handling constraints with negative coefficients
+    ## while not tampering with formulas like y~f(x,-2)
+    st <- yx[[xidx]]
+    st <- gsub("\\-","\\+\\-",gsub("\\+\\-","\\-",st)) ## Convert - to +- (to allow for splitting on '+')
+    ##gsub("[^,]\\-","\\+\\-",st) ## Convert back any - not starting with ','
+    st <- gsub(",\\+",",",st) ## Remove + inside 'f' and 'v' constraints
+    st <- gsub("^\\+","",st) ## Remove leading plus
+    yx[[xidx]] <- st
+
+    ## Match '+' but not when preceeded by ( ... )
+    X <- strsplit(yx[[xidx]],"\\+(?![^\\(]*\\))", perl=TRUE)[[1]]
+    ##regex <- "(?!(\\(*))[\\(\\)]"
+    regex <- "[\\(\\)]"
+    ## Keep squares brackets and |(...) statements
+    ## Extract variables from expressions like
+    ## f(x,b) -> x,b  and  2*x -> 2,cx
+    ## but avoid to tamper with transformation expressions:
+    ## a~(x*b)
+    res <- lapply(X,decomp.specials,regex,pattern2="\\*",pattern.ignore="~",reverse=TRUE,perl=TRUE)
+    ##OLD:
+    ##res <- lapply(X,decomp.specials,pattern2="[*]",reverse=TRUE)
+    xx <- unlist(lapply(res, function(x) x[1]))
+
+    xxf <- lapply(as.list(xx),function(x) decomp.specials(x,NULL,pattern2="\\[|~",perl=TRUE))
+    xs <- unlist(lapply(xxf,function(x) x[1]))
+
+    ## Alter intercepts?
+    intpos <- vapply(xs,function(x) grepl("^[\\-]*[\\.|0-9]+$",x), 0)
+
+    yy <- ys <- NULL
+    if (length(lhs)>0) {
+        yy <- decomp.specials(lhs)
+        yyf <- lapply(yy,function(y) decomp.specials(y,NULL,pattern2="[",fixed=TRUE))
+        ys <- unlist(lapply(yyf,function(x) x[1]))
+    }
+    
+    if (!is.null(object)) {
+      notexo <- c()
+      if (length(lhs)>0) {
+        object <- addvar(object,ys,reindex=FALSE,...)
+        notexo <- ys
+        ## Add link-transformation
+        if (!is.null(invlink)) {
+            if (invlink=="") {
+                object <- transform(object,ys,NULL,post=FALSE)
+                covariance(object,ys) <- NA
+            } else {
+                ff <- function(x) {};  body(ff) <- parse(text=paste0(invlink,"(x)"))
+                object <- transform(object,ys,ff,post=FALSE)
+                covariance(object,ys) <- 0
+            }
+        }
+      }
+
+      if (any(intpos>0)) {
+          if (xs[intpos[1]]=="1") xs[intpos[1]] <- NA
+              intercept(object,ys) <- as.numeric(xs[intpos[1]])
+          xs <- xs[-intpos]          
+      }
+
+        object <- addvar(object,xs,reindex=FALSE ,...)
+        exolist <- c()
+
+        for (i in seq_len(length(xs))) {
+
+            ## Extract transformation statements: var~(expr)
+            xf0 <- strsplit(xx[[i]],"~")[[1]]
+            if (length(xf0)>1) {
+                myexpr <- xf0[2]
+                ftr <- toformula(y="",x=paste0("-1+I(",myexpr,")"))
+                xtr <- all.vars(ftr)
+                xf0 <- xf0[1]
+                transform(object, y=xf0, x=xtr) <- function(x) {
+                    structure(model.matrix(ftr,as.data.frame(x)),dimnames=list(NULL,xf0))
+                }
+            }
+
+            xf <- unlist(strsplit(xf0,"[\\[\\]]",perl=TRUE))
+            if (length(xf)>1) {
+                xpar <- strsplit(xf[2],":")[[1]]
+                if (length(xpar)>1) {
+                    val <- ifelse(xpar[2]=="NA",NA,xpar[2])
+                    valn <- suppressWarnings(as.numeric(val))
+                    covariance(object,xs[i]) <- ifelse(is.na(valn),val,valn)
+                }
+                val <- ifelse(xpar[1]=="NA",NA,xpar[1])
+                valn <- suppressWarnings(as.numeric(val))
+                if (is.na(val) || val!=".") {
+                    intercept(object,xs[i]) <- ifelse(is.na(valn),val,valn)
+                    notexo <- c(notexo,xs[i])
+                }
+            } else { exolist <- c(exolist,xs[i]) }
+        }
+
+        for (i in seq_len(length(ys))) {
+            y <- ys[i]
+            yf <- unlist(strsplit(yy[i],"[\\[\\]]",perl=TRUE))
+            if (length(yf)>1) {
+                ypar <- strsplit(yf[2],":")[[1]]
+                if (length(ypar)>1) {
+                    val <- ifelse(ypar[2]=="NA",NA,ypar[2])
+                    valn <- suppressWarnings(as.numeric(val))
+                    covariance(object,y) <- ifelse(is.na(valn),val,valn)
+                }
+                val <- ifelse(ypar[1]=="NA",NA,ypar[1])
+                valn <- suppressWarnings(as.numeric(val))
+                if (is.na(val) || val!=".") {
+                    intercept(object,y) <- ifelse(is.na(valn),val,valn)
+                }
+            }
+        }
+
+        curvar <- index(object)$var
+        if (exo) {
+            oldexo <- exogenous(object)
+            newexo <- setdiff(exolist,c(notexo,curvar,ys))
+            exogenous(object) <- union(newexo,setdiff(oldexo,notexo))
+        }
+    }
+
+    return(list(object=object,
+                yx=yx,
+                X=X,
+                ys=ys,
+                xx=xx,
+                xs=xs,
+                yy=yy,
+                ys=ys,
+                res=res,
+                notexo=notexo,
+                intpos=intpos,
+                invlink=invlink,
+                lhs=lhs,
+                iscovar=iscovar))
+}
+
+
 ##' Add regression association to latent variable model
 ##'
 ##' Define regression association between variables in a \code{lvm}-object and
 ##' define linear constraints between model equations.
 ##'
-#
 ##'
 ##' The \code{regression} function is used to specify linear associations
 ##' between variables of a latent variable model, and offers formula syntax
@@ -46,7 +213,7 @@
 ##' \code{regression(m,y1~x1+x2) <- list("a1","b1")}
 ##'
 ##' defines \eqn{E(Y_1|X_1,X_2) = a1 X_1 + b1 X_2}. The rhs argument can be a
-##' mixture of character and numeric values (and NA's).
+##' mixture of character and numeric values (and NA's to remove constraints).
 ##'
 ##' The function \code{regression} (called without additional arguments) can be
 ##' used to inspect the linear constraints of a \code{lvm}-object.
@@ -105,6 +272,7 @@
 ##' @export
 "regression<-" <- function(object,...,value) UseMethod("regression<-")
 
+
 ##' @export
 "regression<-.lvm" <- function(object, to=NULL, quick=FALSE, ..., value) {
     dots <- list(...)
@@ -124,20 +292,18 @@
             return(object)
         }
 
-    if (class(value)[1]=="formula") {
-        yx <- lapply(strsplit(as.character(value),"~"),function(x) gsub(" ","",x))[-1]
-        iscovar <- FALSE
-        if (length(yx)==1) {
-            lhs <- NULL; xidx <- 1
-        } else {
-            lhs <- yx[1]; xidx <- 2
-            if (yx[[xidx]][1]=="") {
-                yx[[xidx]] <- yx[[xidx]][-1]
-                iscovar <- TRUE
-            }
-        }
-        X <- strsplit(yx[[xidx]],"+",fixed=TRUE)[[1]]
-        if (iscovar) {
+        if (inherits(value,"formula")) {
+
+            fff <- procformula(object,value,...)
+            object <- fff$object
+            lhs <- fff$lhs
+            xs <- fff$xs
+            ys <- fff$ys
+            res <- fff$res
+            X <- fff$X
+
+            
+        if (fff$iscovar) {
             ## return(covariance(object,var1=decomp.specials(lhs[[1]]),var2=X))
             covariance(object) <- toformula(decomp.specials(lhs[[1]]),X)
             return(object)
@@ -148,76 +314,13 @@
             return(object)
         }
 
-      curvar <- index(object)$var
-      res <- lapply(X,decomp.specials,pattern2="[*]",reverse=TRUE)
-      xx <- unlist(lapply(res, function(x) x[1]))
-
-      notexo <- c()
-      if (length(lhs)>0) {
-        yy <- decomp.specials(lhs)
-        yyf <- lapply(yy,function(y) decomp.specials(y,NULL,pattern2="[",fixed=TRUE))
-        ys <- unlist(lapply(yyf,function(x) x[1]))
-        object <- addvar(object,ys,reindex=FALSE,...)
-        notexo <- ys
-      }
-
-      exo <- c()
-      xxf <- lapply(as.list(xx),function(x) decomp.specials(x,NULL,pattern2="[",fixed=TRUE))
-      xs <- unlist(lapply(xxf,function(x) x[1]))
-
-      ## Remove intercepts?
-      rmint <- na.omit(match("-1",xs))
-      if (length(rmint)>0) intercept(object,ys) <- 0
-      xs <- setdiff(xs,c("-1","1"))
-
-      object <- addvar(object,xs,reindex=FALSE ,...)
-
-      for (i in seq_len(length(xs))) {
-          xf <- unlist(strsplit(xx[[i]],"[\\[\\]]",perl=TRUE))
-          if (length(xf)>1) {
-              xpar <- strsplit(xf[2],":")[[1]]
-              if (length(xpar)>1) {
-                  val <- ifelse(xpar[2]=="NA",NA,xpar[2])
-                  valn <- suppressWarnings(as.numeric(val))
-                  covariance(object,xs[i]) <- ifelse(is.na(valn),val,valn)
-              }          
-              val <- ifelse(xpar[1]=="NA",NA,xpar[1])
-              valn <- suppressWarnings(as.numeric(val))
-              if (is.na(val) || val!=".") {
-                  intercept(object,xs[i]) <- ifelse(is.na(valn),val,valn)
-              notexo <- c(notexo,xs[i])
-              }
-          } else { exo <- c(exo,xs[i]) }
-      }
-
-
-      if (length(lhs)==0) {
-        index(object) <- reindex(object)
-        return(object)
-      }
-
-      if (lava.options()$exogenous) {
-        oldexo <- exogenous(object)
-        newexo <- setdiff(exo,c(notexo,curvar,ys))
-        exogenous(object) <- union(newexo,setdiff(oldexo,notexo))
-      }
-
-      for (i in seq_len(length(ys))) {
-        y <- ys[i]
-        yf <- unlist(strsplit(yy[i],"[\\[\\]]",perl=TRUE))
-        if (length(yf)>1) {
-          ypar <- strsplit(yf[2],":")[[1]]
-          if (length(ypar)>1) {
-            val <- ifelse(ypar[2]=="NA",NA,ypar[2])
-            valn <- suppressWarnings(as.numeric(val))
-            covariance(object,y) <- ifelse(is.na(valn),val,valn)
-          }
-          val <- ifelse(ypar[1]=="NA",NA,ypar[1])
-          valn <- suppressWarnings(as.numeric(val))
-          if (is.na(val) || val!=".") {
-              intercept(object,y) <- ifelse(is.na(valn),val,valn)
-          }
+        if (length(lhs)==0) {
+            index(object) <- reindex(object)
+            return(object)
         }
+
+        for (i in seq_len(length(ys))) {
+        y <- ys[i]
         for (j in seq_len(length(xs))) {
           if (length(res[[j]])>1) {
             regfix(object, to=y[1], from=xs[j],...) <- res[[j]][2]
@@ -226,10 +329,10 @@
           }
         }
       }
-
       object$parpos <- NULL
       return(object)
     }
+
     if (!is.list(value) | length(value)>2) stop("Value should contain names of outcome (to) and predictors (from)")
     if (all(c("to","from")%in%names(value))) {
 
@@ -278,7 +381,7 @@
         if (missing(to)) {
             return(regfix(object))
         }
-        if (class(to)[1]=="formula") {
+        if (inherits(to,"formula")) {
             if (!missing(value)) {
                 regression(object,to,silent=silent,...) <- value
             } else {
@@ -329,3 +432,4 @@
         index(object) <- reindex(object)
         return(object)
     }
+
