@@ -245,7 +245,7 @@ categorical2dummy <- function(x,data,silent=TRUE,...) {
       S <- NULL
       n <- nrow(mydata)
       if (n==1) {
-        S <- diag(ncol(mydata)); colnames(S) <- rownames(S) <- obs
+        S <- diag(nrow=ncol(mydata)); colnames(S) <- rownames(S) <- obs
       }
       if (na.method=="pairwise.complete.obs") {
         mu <- colMeans(mydata,na.rm=TRUE)
@@ -324,26 +324,32 @@ function(M, upper=TRUE) {
 ###{{{ Inverse/pseudo
 
 ##' @export
-Inverse <- function(X,tol=lava.options()$itol,det=TRUE,names=TRUE) {
-  n <- nrow(X)
-  if (nrow(X)==1) {
-    res <- 1/X
-    if (det) attributes(res)$det <- X
+Inverse <- function(X,tol=lava.options()$itol,det=TRUE,names=!chol,chol=FALSE) {
+    n <- nrow(X)
+    if (n==1L) {
+        res <- 1/X
+        if (det) attributes(res)$det <- X
+        if (chol) attributes(res)$chol <- X
+        return(res)
+    }
+    if (chol) {
+        L <- chol(X)
+        res <- chol2inv(L)
+        if (det) attributes(res)$det <- prod(diag(L)^2)
+        if (chol) attributes(res)$chol <- X        
+    } else {
+        svdX <- svd(X)
+        id0 <- numeric(n)
+        idx <- which(svdX$d>tol)
+        id0[idx] <- 1/svdX$d[idx]
+        res <- with(svdX, v%*%diag(id0,nrow=length(id0))%*%t(u))
+        if (det)
+            attributes(res)$det <- prod(svdX$d[svdX$d>tol])
+        attributes(res)$pseudo <- (length(idx)<n)
+        attributes(res)$minSV <- min(svdX$d)
+    }
+    if (names && !is.null(colnames(X))) dimnames(res) <- list(colnames(X),colnames(X))
     return(res)
-  }
-  svdX <- svd(X)
-  id0 <- numeric(n)
-  idx <- which(svdX$d>tol)
-  id0[idx] <- 1/svdX$d[idx]
-  res <- with(svdX, v%*%diag(id0,nrow=length(id0))%*%t(u))
-  if (names && !is.null(colnames(X))) dimnames(res) <- list(colnames(X),colnames(X))
-  if (det)
-      attributes(res)$det <- prod(svdX$d[svdX$d>tol])
-  attributes(res)$pseudo <- (length(idx)<n)
-  attributes(res)$minSV <- min(svdX$d)
-  ##ee <- eigen(X)
-  ##attributes(res)$lambda <- ee$values
-  return(res)
 }
 
 ###}}}
@@ -403,6 +409,7 @@ DFS <- function(M,v,explored=c()) {
   }
   return(explored)
 }
+
 acc <- function(M,v) {
   if (is.character(v)) v <- which(colnames(M)==v)
   colnames(M)[setdiff(DFS(M,v),v)]
@@ -487,8 +494,11 @@ Specials <- function(f,spec,split2="+",...) {
 
 
 ##' @export
-decomp.specials <- function(x,pattern="[()]",pattern2=NULL,sep=",",reverse=FALSE,...) {
-  st <- gsub(" ","",x)
+decomp.specials <- function(x,pattern="[()]",pattern2=NULL, pattern.ignore=NULL, sep=",",reverse=FALSE,...) {
+  st <- gsub(" |^\\(|)$","",x) # Remove white space and leading/trailing parantheses
+  if (!is.null(pattern.ignore)) {
+      if (grepl(pattern.ignore,st,...)) return(st)
+  }
   if (!is.null(pattern)) {
     st <- rev(unlist(strsplit(st,pattern,...)))[1]
   }
@@ -512,3 +522,7 @@ Decomp.specials <- function(x,pattern="[()]") {
   unlist(strsplit(vars,","))
 
 }
+
+printline <- function(n=70) {
+    cat(rep("_", n), "\n", sep="");
+}
